@@ -33,7 +33,6 @@ PATH = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))  # Primary o
 import datetime
 import platform
 import shutil
-from slacker import Slacker
 import re
 import stat
 import subprocess
@@ -42,6 +41,8 @@ import tinys3
 import time
 import traceback
 from github3 import login
+from requests.auth import HTTPBasicAuth
+from requests import post
 
 # Access info class (for version info)
 sys.path.append(os.path.join(PATH, 'src', 'classes'))
@@ -50,8 +51,7 @@ import info
 freeze_command = None
 errors_detected = []
 make_command = "make"
-slack_token = None
-slack_object = None
+zulip_token = None
 s3_access_key = None
 s3_secret_key = None
 s3_connection = None
@@ -68,13 +68,6 @@ log_path = os.path.join(PATH, 'build', 'build-server.log')
 log = open(log_path, 'w+')
 
 
-def run_command(command, working_dir=None):
-    """Utility function to return output from command line"""
-    p = subprocess.Popen(command, shell=True, cwd=working_dir,
-                         stdout=subprocess.PIPE,
-                         stderr=subprocess.STDOUT)
-    return iter(p.stdout.readline, b"")
-
 def output(line):
     """Append output to list and print it"""
     print(line)
@@ -86,6 +79,17 @@ def output(line):
         line += "\n"
     log.write(line)
 
+
+def run_command(command, working_dir=None):
+    """Utility function to return output from command line"""
+    short_command = command.split('" ')[0] # We don't need to print args
+    output("Running %s... (%s)" % (short_command, working_dir))
+    p = subprocess.Popen(command, shell=True, cwd=working_dir,
+                         stdout=subprocess.PIPE,
+                         stderr=subprocess.STDOUT)
+    return iter(p.stdout.readline, b"")
+
+
 def error(line):
     """Append error output to list and print it"""
     print("Error: %s" % line)
@@ -95,6 +99,7 @@ def error(line):
     else:
         log.write(line)
 
+
 def truncate(message, max=256):
     """Truncate the message with ellipses"""
     if len(message) < max:
@@ -102,36 +107,45 @@ def truncate(message, max=256):
     else:
         return "%s..." % message[:max]
 
-def slack(message):
-    """Append a message to slack #build-server channel"""
-    print("Slack: %s" % message)
-    if slack_object:
-        slack_object.chat.post_message("#build-server", truncate(message[:256]))
 
-def slack_upload_log(log, title, comment=None):
-    """Upload a file to slack and notify a slack channel"""
-    # Close log file
-    log.close()
+def zulip_upload_log(log, title, comment=None):
+    """Upload a file to zulip and notify a zulip channel"""
+    output("Zulip Upload: %s" % log_path)
 
-    print("Slack Upload: %s" % log_path)
-    if slack_object:
-        for attempt in range(3):
-            try:
-                # Upload build log to slack (and append platform icon to comment [:linux:, :windows:, or :darwin:])
-                slack_object.files.upload(log_path, filetype="txt", filename="%s-build-server.txt" % platform.system(),
-                                          title=title, initial_comment=':%s: %s' % (platform.system().lower(), comment), channels="#build-server")
-                # Successfully uploaded!
-                break
-            except Exception as ex:
-                # Quietly fail, and try again
-                if attempt < 2:
-                    output("Upload log to Slack failed... trying again")
-                else:
-                    # Throw loud exception
-                    raise Exception('Log upload to Slack failed: %s' % log_path, ex)
+    # Write log file
+    log.flush()
+
+    # Authentication for Zulip
+    zulip_auth = HTTPBasicAuth('builder-bot@openshot.zulipchat.com', zulip_token)
+    filename = "%s-build-server.txt" % platform.system()
+
+    # Upload file to Zulip
+    zulip_url = 'https://openshot.zulipchat.com/api/v1/user_uploads'
+    zulip_upload_url = ''
+    resp = post(zulip_url, data={}, auth=zulip_auth, files={filename: (filename, open(log_path, "rb"))})
+    if resp.ok:
+        zulip_upload_url = resp.json().get("uri", "")
+    print(resp)
+
+    # Determine topic
+    topic = "Successful Builds"
+    if "skull" in comment:
+        topic = "Failed Builds"
+
+    # SEND MESSAGE
+    zulip_url = 'https://openshot.zulipchat.com/api/v1/messages'
+    zulip_data = {
+        "type": "stream",
+        "to": "build-server",
+        "subject": topic,
+        "content": ':%s: %s [Build Log](%s)' % (platform.system().lower(), comment, zulip_upload_url)
+    }
+
+    resp = post(zulip_url, data=zulip_data, auth=zulip_auth)
 
     # Re-open the log (for append)
     log = open(log_path, "a")
+    print(resp)
 
 def get_release(repo, tag_name):
     """Fetch the GitHub release tagged with the given tag and return it
@@ -191,8 +205,7 @@ def parse_version_info(version_path):
 try:
     # Validate command-line arguments
     if len(sys.argv) >= 2:
-        slack_token = sys.argv[1]
-        slack_object = Slacker(slack_token)
+        zulip_token = sys.argv[1]
     if len(sys.argv) >= 4:
         s3_access_key = sys.argv[2]
         s3_secret_key = sys.argv[3]
@@ -237,37 +250,36 @@ try:
     # Get GIT description of openshot-qt-git branch (i.e. v2.0.6-18-ga01a98c)
     openshot_qt_git_desc = ""
     needs_upload = True
-    for line in run_command("git describe --tags"):
-        git_description = line.decode("utf-8").strip()
-        openshot_qt_git_desc = "OpenShot-%s" % git_description
 
-        # Add num of commits from libopenshot and libopenshot-audio (for naming purposes)
-        # If not an official release
-        if git_branch_name == "develop":
-            # Make filename more descriptive for daily builds (add time epoch)
-            openshot_qt_git_desc = "OpenShot-v%s-%d" % (info.VERSION, int(time.time()))
-            openshot_qt_git_desc = "%s-%s-%s" % (openshot_qt_git_desc, version_info.get('libopenshot').get('CI_COMMIT_SHA')[:8], version_info.get('libopenshot-audio').get('CI_COMMIT_SHA')[:8])
-            # Get daily git_release object
-            github_release = get_release(repo, "daily")
-        elif git_branch_name == "release":
-            # Get daily git_release object
-            openshot_qt_git_desc = "OpenShot-v%s-release-candidate-%d" % (info.VERSION, int(time.time()))
-            github_release = get_release(repo, "daily")
-        elif git_branch_name == "master":
-            # Get official version release (i.e. v2.1.0, v2.x.x)
-            github_release = get_release(repo, git_description)
+    # Add num of commits from libopenshot and libopenshot-audio (for naming purposes)
+    # If not an official release
+    if git_branch_name == "develop":
+        # Make filename more descriptive for daily builds (add time epoch)
+        openshot_qt_git_desc = "OpenShot-v%s-%d" % (info.VERSION, int(time.time()))
+        openshot_qt_git_desc = "%s-%s-%s" % (openshot_qt_git_desc, version_info.get('libopenshot').get('CI_COMMIT_SHA')[:8], version_info.get('libopenshot-audio').get('CI_COMMIT_SHA')[:8])
+        # Get daily git_release object
+        github_release = get_release(repo, "daily")
+    elif git_branch_name.startswith("release"):
+        # Get daily git_release object
+        openshot_qt_git_desc = "OpenShot-v%s-release-candidate-%d" % (info.VERSION, int(time.time()))
+        github_release = get_release(repo, "daily")
+    elif git_branch_name == "master":
+        # Get official version release (i.e. v2.1.0, v2.x.x)
+        openshot_qt_git_desc = "OpenShot-v%s" % info.VERSION
+        github_release_name = "v%s" % info.VERSION
+        github_release = get_release(repo, github_release_name)
 
-            # If no release is found, create a new one
-            if not github_release:
-                # Create a new release if one if missing
-                github_release = repo.create_release(git_description, target_commitish="master", prerelease=True)
-        else:
-            # Make filename more descriptive for daily builds
-            openshot_qt_git_desc = "OpenShot-v%s-%d" % (info.VERSION, int(time.time()))
-            openshot_qt_git_desc = "%s-%s-%s" % (openshot_qt_git_desc, version_info.get('libopenshot').get('CI_COMMIT_SHA')[:8], version_info.get('libopenshot-audio').get('CI_COMMIT_SHA')[:8])
-            # Get daily git_release object
-            github_release = get_release(repo, "daily")
-            needs_upload = False
+        # If no release is found, create a new one
+        if not github_release:
+            # Create a new release if one if missing
+            github_release = repo.create_release(github_release_name, target_commitish="master", prerelease=True)
+    else:
+        # Make filename more descriptive for daily builds
+        openshot_qt_git_desc = "OpenShot-v%s-%d" % (info.VERSION, int(time.time()))
+        openshot_qt_git_desc = "%s-%s-%s" % (openshot_qt_git_desc, version_info.get('libopenshot').get('CI_COMMIT_SHA')[:8], version_info.get('libopenshot-audio').get('CI_COMMIT_SHA')[:8])
+        # Get daily git_release object
+        github_release = get_release(repo, "daily")
+        needs_upload = False
 
     # Output git description
     output("git description of openshot-qt-git: %s" % openshot_qt_git_desc)
@@ -319,20 +331,20 @@ try:
         shutil.copyfile("/home/ubuntu/apps/AppImageKit/AppRun", app_run_path)
 
         # Create .desktop file
-        with open(os.path.join(app_dir_path, "openshot-qt.desktop"), "w") as f:
+        with open(os.path.join(app_dir_path, "org.openshot.OpenShot.desktop"), "w") as f:
             f.write('[Desktop Entry]\nName=OpenShot Video Editor\nGenericName=Video Editor\nX-GNOME-FullName=OpenShot Video Editor\nComment=Create and edit amazing videos and movies\nExec=openshot-qt.wrapper %F\nTerminal=false\nIcon=openshot-qt\nType=Application')
 
         # Copy some installation-related files
         shutil.copyfile(os.path.join(PATH, "xdg", "openshot-qt.svg"), os.path.join(app_dir_path, "openshot-qt.svg"))
         shutil.copyfile(os.path.join(PATH, "xdg", "openshot-qt.svg"), os.path.join(app_dir_path, "usr", "share", "pixmaps", "openshot-qt.svg"))
-        shutil.copyfile(os.path.join(PATH, "xdg", "openshot-qt.xml"), os.path.join(app_dir_path, "usr", "share", "mime", "packages", "openshot-qt.xml"))
+        shutil.copyfile(os.path.join(PATH, "xdg", "org.openshot.OpenShot.xml"), os.path.join(app_dir_path, "usr", "share", "mime", "packages", "org.openshot.OpenShot.xml"))
         shutil.copyfile(os.path.join(PATH, "xdg", "openshot-qt"), os.path.join(app_dir_path, "usr", "lib", "mime", "packages", "openshot-qt"))
 
         # Copy the entire frozen app
         shutil.copytree(os.path.join(PATH, "build", exe_dir), os.path.join(app_dir_path, "usr", "bin"))
 
         # Copy desktop integration wrapper (prompts users to install shortcut)
-        launcher_path = os.path.join(app_dir_path, "usr", "bin", "openshot-qt")
+        launcher_path = os.path.join(app_dir_path, "usr", "bin", "openshot-qt-launch")
         os.rename(os.path.join(app_dir_path, "usr", "bin", "launch-linux.sh"), launcher_path)
         desktop_wrapper = os.path.join(app_dir_path, "usr", "bin", "openshot-qt.wrapper")
         shutil.copyfile("/home/ubuntu/apps/AppImageKit/desktopintegration", desktop_wrapper)
@@ -390,17 +402,47 @@ try:
     if platform.system() == "Windows":
 
         # Move python folder structure, since Cx_Freeze doesn't put it in the correct place
-        exe_dir = os.path.join(PATH, 'build', 'exe.mingw-3.6')
-        python_dir = os.path.join(exe_dir, 'lib', 'python3.6')
-        if not os.path.exists(python_dir):
-            os.mkdir(python_dir)
+        exe_dir = os.path.join(PATH, 'build', 'exe.mingw-3.7')
+        python_dir = os.path.join(exe_dir, 'lib', 'python3.7')
 
-            # Copy all non-zip files from /lib/ into /python3.X/
-            for lib_file in os.listdir(os.path.join(exe_dir, 'lib')):
-                if not ".zip" in lib_file and not lib_file == "python3.6":
-                    lib_src_path = os.path.join(os.path.join(exe_dir, 'lib'), lib_file)
-                    lib_dst_path = os.path.join(os.path.join(python_dir), lib_file)
-                    shutil.move(lib_src_path, lib_dst_path)
+        # Remove a redundant openshot_qt module folder (duplicates lots of files)
+        duplicate_openshot_qt_path = os.path.join(python_dir, 'openshot_qt')
+        if os.path.exists(duplicate_openshot_qt_path):
+            shutil.rmtree(duplicate_openshot_qt_path, True)
+
+        # Remove the following paths. cx_Freeze is including many unneeded files. This prunes them out.
+        paths_to_delete = ['mediaservice', 'imageformats', 'platforms', 'printsupport', 'lib/openshot_qt', 'resvg.dll']
+        for delete_path in paths_to_delete:
+            full_delete_path = os.path.join(exe_dir, delete_path)
+            output("Delete path: %s" % full_delete_path)
+            if os.path.exists(full_delete_path):
+                if os.path.isdir(full_delete_path):
+                    # Delete Folder
+                    shutil.rmtree(full_delete_path)
+                else:
+                    # Delete File
+                    os.unlink(full_delete_path)
+            else:
+                output("Invalid delete path: %s" % full_delete_path)
+
+        # Replace these folders (cx_Freeze messes this up, so this fixes it)
+        paths_to_replace = ['imageformats', 'platforms']
+        for replace_name in paths_to_replace:
+            if windows_32bit:
+                shutil.copytree(os.path.join('C:\\msys32\\mingw32\\share\\qt5\\plugins', replace_name), os.path.join(exe_dir, replace_name))
+            else:
+                shutil.copytree(os.path.join('C:\\msys64\\mingw64\\share\\qt5\\plugins', replace_name), os.path.join(exe_dir, replace_name))
+
+        # Copy Qt5Core.dll, Qt5Svg.dll to root of frozen directory
+        paths_to_copy = [("Qt5Core.dll", "C:\\msys64\\mingw64\\bin\\"), ("Qt5Svg.dll", "C:\\msys64\\mingw64\\bin\\")]
+        if windows_32bit:
+            paths_to_copy = [("Qt5Core.dll", "C:\\msys32\\mingw32\\bin\\"), ("Qt5Svg.dll", "C:\\msys32\\mingw32\\bin\\")]
+        for qt_file_name, qt_parent_path in paths_to_copy:
+            qt5_path = os.path.join(qt_parent_path, qt_file_name)
+            new_qt5_path = os.path.join(exe_dir, qt_file_name)
+            if os.path.exists(qt5_path) and not os.path.exists(new_qt5_path):
+                output("Copying %s to %s" % (qt5_path, new_qt5_path))
+                shutil.copy(qt5_path, new_qt5_path)
 
         # Delete debug Qt libraries (since they are not needed, and cx_Freeze grabs them)
         for sub_folder in ['', 'platforms', 'imageformats']:
@@ -416,13 +458,30 @@ try:
         if windows_32bit:
             only_64_bit = ""
 
+        # Add version metadata to frozen app launcher
+        launcher_exe = os.path.join(exe_dir, "openshot-qt.exe")
+        verpatch_success = True
+        verpatch_command = '"verpatch.exe" "{}" /va /high "{}" /pv "{}" /s product "{}" /s company "{}" /s copyright "{}" /s desc "{}"'.format(launcher_exe, info.VERSION, info.VERSION, info.PRODUCT_NAME, info.COMPANY_NAME, info.COPYRIGHT, info.PRODUCT_NAME)
+        verpatch_output = ""
+        # version-stamp executable
+        for line in run_command(verpatch_command):
+            output(line)
+            if line:
+                verpatch_success = False
+                verpatch_output = line
+
+        # Was the verpatch command successful
+        if not verpatch_success:
+            # Verpatch failed (not fatal)
+            error("Verpatch Error: Had output when none was expected (%s)" % verpatch_output)
+
         # Copy uninstall files into build folder
         for file in os.listdir(os.path.join("c:/", "InnoSetup")):
             shutil.copyfile(os.path.join("c:/", "InnoSetup", file), os.path.join(PATH, "build", file))
 
         # Create Installer (OpenShot-%s-x86_64.exe)
         inno_success = True
-        inno_command = '"C:\Program Files (x86)\Inno Setup 5\iscc.exe" /Q /DVERSION=%s /DONLY_64_BIT=%s "%s"' % (version, only_64_bit, os.path.join(PATH, 'installer', 'windows-installer.iss'))
+        inno_command = '"iscc.exe" /Q /DVERSION=%s /DONLY_64_BIT=%s "%s"' % (version, only_64_bit, os.path.join(PATH, 'installer', 'windows-installer.iss'))
         inno_output = ""
         # Compile Inno installer
         for line in run_command(inno_command):
@@ -445,7 +504,7 @@ try:
 
         # Sign the installer
         key_sign_success = True
-        key_sign_command = '"C:\\Program Files (x86)\\kSign\\kSignCMD.exe" /f "%s" /p "%s" /d "OpenShot Video Editor" /du "http://www.openshot.org" "%s"' % (windows_key, windows_key_password, app_build_path)
+        key_sign_command = '"kSignCMD.exe" /f "%s%s" /p "%s" /d "OpenShot Video Editor" /du "http://www.openshot.org" "%s"' % (windows_key, only_64_bit, windows_key_password, app_build_path)
         key_sign_output = ""
         # Sign MSI
         for line in run_command(key_sign_command):
@@ -493,16 +552,17 @@ try:
             # Torrent succeeded! Upload the torrent to github
             url = upload(torrent_path, github_release)
 
-            # Notify Slack
-            slack_upload_log(log, "%s: Build logs for %s" % (platform.system(), app_name), "Successful *%s* build: %s" % (git_branch_name, download_url))
+            # Notify Zulip
+            zulip_upload_log(log, "%s: Build logs for %s" % (platform.system(), app_name), "Successful *%s* build: %s" % (git_branch_name, download_url))
 
 except Exception as ex:
     tb = traceback.format_exc()
     error("Unhandled exception: %s - %s" % (str(ex), str(tb)))
 
-
-# Report any errors detected
-if errors_detected:
-    slack_upload_log(log, "%s: Error log for *%s* build" % (platform.system(), git_branch_name), ":skull_and_crossbones: %s" % truncate(errors_detected[0], 150))
+if not errors_detected:
+    output("Successfully completed build-server script!")
+else:
+    # Report any errors detected
+    output("build-server script failed!")
+    zulip_upload_log(log, "%s: Error log for *%s* build" % (platform.system(), git_branch_name), ":skull_and_crossbones: %s" % truncate(errors_detected[0], 100))
     exit(1)
-
